#include <allegro.h>
#include <stdio.h>
#include <string.h>

#include "base.h"
#include "resources.h"
#include "makecol.h"
#include "macros.h"


char *bname[BUILDINGS]={"ops center","solargen(X)","fission reactor(X)","fusion reactor(X)","habitat","hydroponic farm","waste recycler","laboratory(X)","fabricator(X)",
	"chemical plant","refinery(X)","miner","air processor","storage","landing pad(X)","X-node"};
#include "bdescs.h"

// Stolen from Allegro source, whole new ctext_xleft proc :)
int d_ctext_xleft_proc(int msg, DIALOG *d, int c)
{
   if (msg==MSG_DRAW) {
      int rtm;
      int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
      FONT *oldfont = font;

      if (d->dp2)
	 font =(FONT*) d->dp2;

      rtm = text_mode(d->bg);
      gui_textout(screen, (const char*)d->dp, d->x+d->w/2, d->y, fg, TRUE);
      text_mode(rtm);

      font = oldfont;
   }

   return D_O_K;
}


// --------- Building -----------
Building::Building(Base *base,Building *paren,int type)
{
	this->base=base;
	this->type=type;

	parent=paren;
	edges[0]=edges[1]=edges[2]=edges[3]=NULL;

	switch(type)
	{
	case B_STORAGE:
		base->store->limit+=10000;
		break;
	}

}
Building::~Building()
{
	switch(type)
	{
	case B_STORAGE:
		base->store->limit-=10000;
		break;
	}
}
void Building::build(int conn,int type)
{
	switch(type)
	{
	case B_HYDRO:
		edges[conn]=new Hydro(base,this,type);
		break;
	case B_CHEM:
		edges[conn]=new Chemplant(base,this,type);
		break;
	default:
		edges[conn]=new Building(base,this,type);
		break;
	}
}
char *Building::getstring()
{
	return bname[type];
}

void Building::update()
{
	bool succ;
	switch(type)
	{
	case B_RECYCLER:
		succ=base->store->remove(R_WASTE,41.0/24.0);
		if(succ)
		{
			base->store->add(R_WATER,25.0/24.0);
			base->store->add(R_NUTRIENTS,10.0/24.0);
		}
		break;
	case B_AIRPROC:
		// Process 5 000 000 liters of air a day
		//base->store->add(R_CO2,(0.0063*5000000.0)/24.0);
		base->store->add(R_WATER,(0.000002*5000000.0)/24.0);
		base->store->add(R_OXYGEN,(0.000010*5000000.0)/24.0);
		break;
	case B_MINER:
		succ=base->store->enough(R_METHANE,40.0/24.0) &&
			 base->store->enough(R_OXYGEN,50.0/24.0);
		if(succ)
		{
			base->store->remove(R_METHANE,40.0/24.0);
			base->store->remove(R_OXYGEN,50.0/24.0);

			base->store->add(R_ALUMINIUM,20/24.0);
			base->store->add(R_IRON,40/24.0);
			base->store->add(R_SODIUM,13/24.0);
			base->store->add(R_PHOSPHORUS,14/24.0);
			base->store->add(R_SILICON,28/24.0);
			base->store->add(R_TITANIUM,7/24.0);
			//base->store->add(R_CHROMIUM,20/24.0); No such metal, oops
			base->store->add(R_POTASSIUM,5/24.0);
			base->store->add(R_MANGANESE,4/24.0);
		}
		break;
	default:
		// Default case, do nothing.
		break;
	}
}

char *build_dlg_lbox(int index, int *list_size)
{
	if(index<0)
	{
		*list_size=BUILDINGS;
		return NULL;
	}
	return bname[index];
}
#define DX	212
#define DY	200
#define BX	620
#define BY	325
DIALOG build_dlg[] =
{
	/*     (dialog proc)     (x)       (y)   (w)   (h)    (fg)  (bg)  (key)	(flags)     (d1)  (d2)  (dp) */
	/*0*/{ d_box_proc,		    DX,     DY,  600,  300,   0,	0,		0,	0,          0,    0,    NULL },	
	/*1*/{ d_button_proc,    BX-60,  BY-60,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Build" },
	/*2*/{ d_button_proc,    BX+40,  BY-10,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Build" },
	/*3*/{ d_button_proc,    BX-60,  BY+40,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Build" },
    /*4*/{ d_button_proc,   BX-160,  BY-10,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Build" },
	/*5*/{ d_button_proc,   DX+460, DY+260,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Cancel" },

	/*6*/{ d_list_proc,		 DX+20,  DY+60,	 200,	150,  0,	0,		0,	0,			0,	   0,	build_dlg_lbox},

	/*X*/{ d_text_proc,		 DX+20,	 DY+40,	 200,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"Select module to build"},
	/*X*/{ d_box_proc,	     BX-25,  BY-25,  50,    50,	  0,	0,		0,	0,          0,     0,   NULL},
	/*X*/{ d_text_proc,	 	 BX-4,    BY-5,   8,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"X"	},	

	/*X*/{ d_yield_proc },
	/*X*/{ NULL,              0,    0,    0,    0,    0,    0,    0,    0,			0,    0,    NULL }
};
void Building::build_dialog()
{	
	int choise=0;
	for(int n=0;build_dlg[n].proc!=NULL;n++)
		build_dlg[n].fg=WHITE;
	for(int n=0;n<4;n++)
	{
		if(edges[n]!=NULL)
		{
			build_dlg[n+1].proc=*d_ctext_xleft_proc;
			build_dlg[n+1].dp=(void*)bname[edges[n]->type];
			build_dlg[n+1].flags=0;
		}
		else
		{
			build_dlg[n+1].proc=*d_button_proc;
			build_dlg[n+1].dp=(void*)"Build";
			build_dlg[n+1].flags=D_EXIT;
		}
	}
	choise=popup_dialog(build_dlg,5);
	if(choise>0 && choise<5)
	{
		int type=build_dlg[6].d1;
		build(choise-1,type);
	}
}
DIALOG prop_dlg[] =
{
	/*     (dialog proc)     (x)       (y)   (w)   (h)    (fg)  (bg)  (key)	(flags)     (d1)  (d2)  (dp) */
	/*0*/{ d_box_proc,		    DX,     DY,  600,  300,   0,	0,		0,	0,          0,    0,    NULL },	
	/*1*/{ d_button_proc,   DX+460, DY+270,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Close" },
	/*2*/{ d_button_proc,   DX+580,  DY+10,   14,   14,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"X" },

	/*3*/{ d_text_proc,		 DX+20,	 DY+20,	 200,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"Module"},
	/*4*/{ d_textbox_proc,	 DX+20,  DY+40,  560,  200,	  0,	0,		0,	0,			0,	   0,	(void*)"X"	},	

	/*X*/{ d_yield_proc },
	/*X*/{ NULL,                  0,    0,    0,    0,    0,    0,      0,  0,			0,    0,    NULL }
};

void Building::properties_dialog()
{
	for(int n=0;prop_dlg[n].proc!=NULL;n++)
		prop_dlg[n].fg=WHITE;

	prop_dlg[3].dp=(void*)bname[type];
	prop_dlg[4].dp=(void*)bdesc[type];

	popup_dialog(prop_dlg,2);
}

// ------------------ Hydro ----------------------
Hydro::Hydro(Base *base,Building *paren,int type):Building(base,paren,type)
{
	crop=0;
	malnutrition=0;
}
Hydro::~Hydro() {}
char *Hydro::getstring()
{
	if(crop>=0)
		if(malnutrition>0)
			sprintf(buffer,"%s (Malnutrition!)",bname[type]);
		else
			sprintf(buffer,"%s (%i%% grown)",bname[type],(crop*100)/168);
	else 
		sprintf(buffer,"%s (Crop death!)",bname[type]);
	return buffer;
}
void Hydro::update()
{
	bool succ;
	succ=base->store->remove(R_NUTRIENTS,8.0/24.0);
	succ=succ && base->store->remove(R_CO2,2.0/24.0);
	if(succ) 
	{
		malnutrition=0;
		crop++; // Grow crops
		base->store->add(R_OXYGEN,1.0/24.0);
	}
	else 
	{
		malnutrition++;
		if(malnutrition>12 && rand()%1000<150 && crop>=0) 
		{
			crop=-1; 
			printf("Hydroponic crops died from lack of nutrients!");
		} 
		else printf("Not enough nutrients to sustain crop growth!");
	}
	
	if(crop==168)
	{
		crop=0;
		double cropmult=(rand()%800)/800.0+0.4;
		base->store->add(R_FOOD,7.0*7.0*cropmult);
		base->store->add(R_WASTE,1.0*7.0*cropmult);
	}

}
DIALOG hydro_prop_dlg[] =
{
	/*     (dialog proc)     (x)       (y)   (w)   (h)    (fg)  (bg)  (key)	(flags)     (d1)  (d2)  (dp) */
	/*0*/{ d_box_proc,		    DX,     DY,  600,  300,   0,	0,		0,	0,          0,    0,    NULL },	
	/*1*/{ d_button_proc,   DX+460, DY+270,  120,   20,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"Close" },
	/*2*/{ d_button_proc,   DX+580,  DY+10,   14,   14,   0,	0,		0,	D_EXIT,		0,	   0,   (void*)"X" },

	/*3*/{ d_text_proc,		 DX+20,	 DY+20,	 200,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"Module"},
	/*4*/{ d_textbox_proc,	 DX+20, DY+100,  560,  140,	  0,	0,		0,	0,			0,	   0,	(void*)"X"	},	

	/*5*/{ d_box_proc,		 DX+20,  DY+60,  200,   20,   0,    0,      0,  0,          0,    0,    NULL },
	/*6*/{ d_box_proc,		 DX+20,  DY+60,    0,   20,   0,    0,      0,  0,          0,    0,    NULL },

	/*7*/{ d_text_proc,		 DX+240, DY+64,	 200,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"X"},
	/*8*/{ d_text_proc,		 DX+20,	 DY+45,	 200,	12,	  0,	0,		0,	0,			0,	   0,	(void*)"Growth cycle"},

	/*X*/{ d_yield_proc },
	/*X*/{ NULL,                  0,    0,    0,    0,    0,    0,      0,  0,			0,    0,    NULL }
};

void Hydro::properties_dialog()
{
	char tmp[10];
	for(int n=0;hydro_prop_dlg[n].proc!=NULL;n++)
		hydro_prop_dlg[n].fg=WHITE;

	hydro_prop_dlg[3].dp=(void*)bname[type];
	hydro_prop_dlg[4].dp=(void*)bdesc[type];

	hydro_prop_dlg[6].w=(crop*200)/168;
	hydro_prop_dlg[6].bg=makecol(0,128,0);

	sprintf(tmp,"%i%%",(crop*100)/168);
	hydro_prop_dlg[7].dp=(void*)tmp;

	popup_dialog(hydro_prop_dlg,2);
}

Chemplant::Chemplant(Base *base,Building *paren,int type):Building(base,paren,type)
{
	reaction=0;
}
Chemplant::~Chemplant() {}

char *reactions[7]={"idle","sabatier","H2O electrolysis","direct CO2 reduction","reverse water-gas shift","CH4 pyrolysis","CO2+C reduction"};
char *Chemplant::getstring()
{
	sprintf(buffer,"%s (%s)",bname[type],reactions[reaction]);
	return buffer;
}

void Chemplant::update()
{
	bool succ;
	switch(reaction)
	{
	case 1:
		// Sabatier
		succ=base->store->enough(R_CO2,20.0/24.0) &&
			 base->store->enough(R_HYDROGEN,1.428/24.0);
		if(succ)
		{
			base->store->remove(R_CO2,20.0/24.0);
			base->store->remove(R_HYDROGEN,1.428/24.0);
			base->store->add(R_METHANE,21.42/24.0);
		}
		break;
	case 2:
		// Water electrolysis
		succ=base->store->remove(R_WATER,20.0/24.0);
		if(succ)
		{
			base->store->add(R_OXYGEN,17.7/24.0);
			base->store->add(R_HYDROGEN,2.2/24.0);
		}
		break;
	case 3:
		// Direct CO2 reduction
		succ=base->store->remove(R_CO2,6.0/24.0);
		if(succ)
		{
			base->store->add(R_CO,3.9/24.0);
			base->store->add(R_OXYGEN,2.1/24.0);
		}
		break;
	case 4:
		// Reverse water-gas shift
		succ=base->store->enough(R_CO2,20.0/24.0) &&
			 base->store->enough(R_HYDROGEN,1.428/24.0);
		if(succ)
		{
			base->store->remove(R_CO2,20.0/24.0);
			base->store->remove(R_HYDROGEN,0.714/24.0);
			base->store->add(R_CO,13.0/24.0);
			base->store->add(R_WATER,6.66/24.0);
		}
		break;
	case 5:
		// CH4 pyrolysis
		succ=base->store->remove(R_METHANE,20.0/24.0);
		if(succ)
		{
			base->store->add(R_CARBON,17.8/24.0);
			base->store->add(R_HYDROGEN,2.2/24.0);
		}
		break;
	case 6:
		// CO2+C reduction
		succ=base->store->enough(R_CO2,30.0/24.0) &&
			 base->store->enough(R_CARBON,10.0/24.0);
		if(succ)
		{
			base->store->remove(R_CO2,30.0/24.0);
			base->store->remove(R_CARBON,10.0/24.0);
			base->store->add(R_CO,40.0/24.0);
		}
		break;
	default:
		// Idle, do nothing.
		break;
	}

}

char *chem_dlg_lbox(int index, int *list_size)
{
	if(index<0)
	{
		*list_size=7;
		return NULL;
	}
	return reactions[index];
}
DIALOG chem_prop_dlg[] =
{
	/*     (dialog proc)     (x)       (y)   (w)   (h)    (fg)  (bg)  (key)	(flags)     (d1)  (d2)  (dp) */
	/*0*/{ d_box_proc,		    DX,     DY,  600,  300,   0,	0,		0,	0,          0,    0,    NULL },	
	/*1*/{ d_button_proc,   DX+460, DY+270,  120,   20,   0,	0,		0,	D_EXIT,		0,	  0,   (void*)"Close" },
	/*2*/{ d_button_proc,   DX+580,  DY+10,   14,   14,   0,	0,		0,	D_EXIT,		0,	  0,   (void*)"X" },

	/*3*/{ d_text_proc,		 DX+20,	 DY+20,	 200,	12,	  0,	0,		0,	0,			0,	  0,	(void*)"Module"},
	/*4*/{ d_textbox_proc,	 DX+20, DY+150,  560,   90,	  0,	0,		0,	0,			0,	  0,	(void*)"X"	},	

	/*5*/{ d_text_proc,		 DX+20,	 DY+40,	 200,	12,	  0,	0,		0,	0,			0,	  0,	(void*)"Active reaction"},
	/*6*/{ d_list_proc,		 DX+20,  DY+50,	 300,	70,   0,	0,		0,	0,			0,	  0,	chem_dlg_lbox},

	/*X*/{ d_yield_proc },
	/*X*/{ NULL,                  0,    0,    0,    0,    0,    0,      0,  0,			0,    0,    NULL }
};

void Chemplant::properties_dialog()
{
	for(int n=0;chem_prop_dlg[n].proc!=NULL;n++)
		chem_prop_dlg[n].fg=WHITE;

	chem_prop_dlg[3].dp=(void*)bname[type];
	chem_prop_dlg[4].dp=(void*)bdesc[type];

	chem_prop_dlg[6].d1=reaction;

	popup_dialog(chem_prop_dlg,2);

	reaction=chem_prop_dlg[6].d1;
}

// --------- Crewman ----------
Crewman::Crewman(Base *base)
{
	this->base=base;
	active=true;
	water=food=0;
}
Crewman::~Crewman() {}

void Crewman::update()
{
	if(active) 
	{
		bool succ;
		succ=base->store->remove(R_FOOD,   0.6  / 24.0);
		succ=succ && base->store->remove(R_WATER,  4.0  / 24.0);
		if(succ==false) 
			printf("Out of food/water!\n");
		else
		{
			base->store->add(R_WASTE, 4.0 / 24.0);
		}
	
		succ=base->store->remove(R_OXYGEN, 0.84 / 24.0);
		if(succ==false) 
		{
			printf("DEAD!\n");
			active=false;
		}
		else
		{
			base->store->add(R_CO2, 0.84 / 24.0);
		}
	}
}



// --------- Base -----------
Base::Base(double lat,double lon)
{
	this->lat=lat;
	this->lon=lon;
	strcpy(name,"Sagan Station");

	store=new Storage();

	num_crew=10;
	max_crew=20;

	for(int n=0;n<num_crew;n++)
		crew.push_back(new Crewman(this));

	// Create some initial buildings:
	root=new Building(this,NULL,B_OPS);
	root->edges[0]=new Building(this,root,B_PAD);
	
	root->edges[1]=new Building(this,root,B_HAB);
	root->edges[1]->edges[0]=new Hydro(this,root->edges[1],B_HYDRO);
	root->edges[1]->edges[1]=new Building(this,root->edges[1],B_RECYCLER);

	root->edges[2]=new Building(this,root,B_AIRPROC);
	root->edges[2]->edges[0]=new Chemplant(this,root->edges[2],B_CHEM);
	root->edges[2]->edges[1]=new Chemplant(this,root->edges[2],B_CHEM);

	root->edges[3]=new Building(this,root,B_STORAGE);

}

Base::~Base()
{


}


void Base::recurse(Building *building)
{
	building->update();
	for(int n=0;n<4;n++)
	{
		if(building->edges[n]!=NULL) recurse(building->edges[n]);
	}
}
void Base::update(int curtime)
{
	if(curtime%24==0) store->day_begin();

	// Update the crew status.
	for(vector<Crewman*>::iterator crewman=crew.begin();crewman!=crew.end();)
	{
		(*crewman)->update();
		if(!(*crewman)->active) crewman=crew.erase(crewman);
		else crewman++;
	}
	recurse(root);
}
