#define USE_CONSOLE
#include <allegro.h>
#include <stdio.h>

#include "base.h"
#include "resources.h"
#include "makecol.h"
#include "macros.h"

BITMAP *mars;
BITMAP *screen2;

Base *base;
int curtime=0;

// Functions
volatile bool exit_flag=false;

volatile bool logic_update;
void logic_timer()
{
	logic_update=true;
}
END_OF_FUNCTION(logic_timer);

class Listbox
{
public:
	Building *selection;

	Listbox(int x,int y,int w,int h);
	~Listbox() {};
	
	void draw(BITMAP *bmp);
	void mclick();
	void mwheel(int z);

private:
	BITMAP *buffer;
	BITMAP *buttons;
	BITMAP *icons;
	int scrolled;
	int cy,sy; // Currect Y, selected element Y
	int x,y;

	void recurse(Building *node,int level);
};
Listbox::Listbox(int x,int y,int w,int h)
{
	cy=0;
	sy=-1;
	selection=NULL;
	buffer=create_bitmap(w,h);
	buttons=load_bitmap("lbox1.bmp",NULL);
	icons=load_bitmap("lbox2.bmp",NULL);
	this->x=x;
	this->y=y;
	scrolled=0;
}
void Listbox::draw(BITMAP *bmp)
{
	cy=0;
	clear_to_color(buffer,0);
	recurse(base->root,0);
	if(bmp==screen) scare_mouse();
	blit(icons,buffer,0,0,0,buffer->h-icons->h,icons->w,icons->h);
	blit(buttons,buffer,0,0,buffer->w-buttons->w,0,buttons->w,buttons->h);
	blit(buffer,bmp,0,0,x,y,buffer->w,buffer->h);
	unscare_mouse();
}
void Listbox::recurse(Building *node,int level)
{
	if(cy==sy) 
	{
		selection=node;
		rectfill(buffer,0,(cy-scrolled)*10-1,buffer->w,(cy-scrolled)*10+7,makecol(0,0,128));
	}
	textprintf(buffer,font,level*16,(cy-scrolled)*10,WHITE,"%s",node->getstring());

	cy++;
	for(int n=0;n<4;n++)
		if(node->edges[n]!=NULL)
			recurse(node->edges[n],level+1);
}
void Listbox::mwheel(int z)
{
	int mx=mouse_x-x;
	int my=mouse_y-y;

	if(mx>0 && mx<buffer->w && my>0 && my<buffer->h)
	{
		if(z>0) scrolled--;
		if(z<0) scrolled++;
		if(scrolled<0) scrolled=0;
		draw(screen);
	}
}
void Listbox::mclick()
{
	int mx=mouse_x-x;
	int my=mouse_y-y;

	if(mx>0 && mx<buffer->w && my>0 && my<buffer->h)
	{
		if(mx<buffer->w-25)
		{
			if(my<buffer->h-25)
			{
				sy=(my/10)+scrolled;
				printf("Mclick hit! %i\n",sy);
			}
			else
			{
				// We hit the icons!
				int i=mx/25;
				switch(i)
				{
				case 0: // properties
					if(selection!=NULL) selection->properties_dialog();
					break;
				case 1: // build new module
					if(selection!=NULL) selection->build_dialog();
					break;
				case 2: // remove module
					if(selection!=base->root && selection!=NULL)  // Base root node or NULL node can't be deleted! Bad things would happen in code...
					{
						if(selection->edges[0]==NULL && selection->edges[1]==NULL &&
							selection->edges[2]==NULL && selection->edges[3]==NULL)  // Can't delete anything with children
						{
							for(int n=0;n<4;n++)
							{
								if(selection->parent->edges[n]==selection)
									selection->parent->edges[n]=NULL;
							}
							delete selection;
						}
					}
					break;
				}
			}
		}
		else
		{
			// We hit the buttons!
			if(my<=78) scrolled--;
			else scrolled++;
	
			if(scrolled<0) scrolled=0;

			printf("Mclick scroll!");
		}
		draw(screen);
	}
}

Listbox *lbox;

Building *base_element()
{
	int x=mouse_x;
	int y=mouse_y;

	if(x>700 && x<1024 && y>590 && y<768)
	{
//		baserecurse(screen2,base->root,0);
	}
	return NULL;
}

void mapscreen()
{
	clear_to_color(screen2,0);
	blit(mars,screen2,0,0,0,0,mars->w,mars->h);
	
	double x,y;
	x=base->lon; y=-base->lat;
	
	x=((x/180.0)*mars->w/2)*cos(y);
	y=(y/90.0)*mars->h/2;

	circle(screen2,x+mars->w/2,y+mars->h/2,5,makecol(255,255,255));

	textprintf(screen2,font,10,550,WHITE,"Station 1: %s",base->name);
	textprintf(screen2,font,10,560,WHITE,"Lat: %.2f; Lon: %.2f",base->lat,base->lon);

	textprintf(screen2,font,10,580,WHITE,"Crew: %i",base->crew.size());

	textprintf(screen2,font,350,550,WHITE,"Day %i, %.2i:00",(curtime/24)+1,curtime%24);

	textprintf(screen2,font,170,590,WHITE,"kg");
	textprintf(screen2,font,260,590,WHITE,"Flow");
	textprintf(screen2,font,520,590,WHITE,"kg");
	textprintf(screen2,font,610,590,WHITE,"Flow");
	for(int n=0;n<15;n++)
	{
		hline(screen2,10,608+n*10,320,GREY50);
		textprintf(screen2,font,10,600+n*10,WHITE,"%s",resname[n]);
		textprintf(screen2,font,170,600+n*10,WHITE,"%.2f",base->store->get(n));
		textprintf(screen2,font,260,600+n*10,WHITE,"%.2f",base->store->getdelta(n));
	}
	for(int n=15;n<ELEMENTS;n++)
	{
		hline(screen2,350,608+(n-15)*10,670,GREY50);
		textprintf(screen2,font,350,600+(n-15)*10,WHITE,"%s",resname[n]);
		textprintf(screen2,font,520,600+(n-15)*10,WHITE,"%.2f",base->store->get(n));
		textprintf(screen2,font,610,600+(n-15)*10,WHITE,"%.2f",base->store->getdelta(n));
	}
	textprintf(screen2,font,350,730,WHITE,"Total: %.2f kg",base->store->total);
	textprintf(screen2,font,350,740,WHITE,"Space: %.2f kg",base->store->limit);

	lbox->draw(screen2);

	scare_mouse();
	blit(screen2,screen,0,0,0,0,SCREEN_W,SCREEN_H);
	unscare_mouse();
}


void exitproc()
{
	exit_flag=true;
}
int main() 
{
	srand(time(NULL));

	allegro_init();
	install_keyboard();
	install_timer();
	install_mouse();

	LOCK_VARIABLE(logic_update);
	LOCK_FUNCTION(logic_timer);
	//install_int(logic_timer,2500);
	install_int(logic_timer,1000);
	
	set_color_depth(32);
	set_gfx_mode(GFX_AUTODETECT/*_WINDOWED*/,1024,768,0,0);

	set_window_close_hook(exitproc);
	text_mode(-1);

	set_color_conversion(COLORCONV_TOTAL);
	mars=load_bitmap("mars.bmp",NULL);
	screen2=create_bitmap(SCREEN_W,SCREEN_H);

	show_mouse(screen);



	base=new Base(19.33,-33.25);
	
	lbox=new Listbox(700,590,314,158);

	mapscreen();
	//baseview();

	int mb=0,mz=0;
	char ch=0;
	while(ch!=KEY_ESC && !exit_flag)
	{
		while(!logic_update) 
		{
			yield_timeslice();
			if(exit_flag) logic_update=true;

			if(!mouse_b && mb) lbox->mclick();
			if(mz!=mouse_z) lbox->mwheel(mouse_z-mz);
			mb=mouse_b; mz=mouse_z;
			if(keypressed()) ch=readkey() >> 8;
		}
		logic_update=false;

		curtime++;
		base->update(curtime);
		mapscreen();
		//baseview();

	}

	return 0;
}
END_OF_MAIN()