#ifndef RESOURCES_H
#define RESOURCES_H

// Raw elements
#define R_ALUMINIUM		0
#define R_ARGON			1
#define R_CARBON		2
#define R_CHLORINE		3
#define R_IRON			4
#define R_MAGNESIUM		5
#define R_MANGANESE		6
#define R_NITROGEN		7
#define R_PHOSPHORUS	8
#define R_POTASSIUM		9
#define R_SILICON		10
#define R_SODIUM		11
#define R_SULPHUR		12
#define R_TITANIUM		13
#define R_URANIUM		14

// Compounds
#define R_HYDROGEN		15
#define R_OXYGEN		16
#define R_METHANE		17
#define R_WATER			18
#define R_CO2			19
#define R_CO			20
#define R_NUTRIENTS		21
#define R_FOOD			22
#define R_WASTE			23

#define ELEMENTS	24

extern const char *resname[];  // String of resource names

class Storage
{
public:

	double res[ELEMENTS];	// Kilograms of resource
	double total;			// Total amount in storage
	double limit;			// How much can be stored

	Storage();
	~Storage();

	void day_begin(); // called at start of each new day.
	
	double add(int n,double amount);  // Add element to storage (if returns positive value, that much didn't fit storage!)
	bool remove(int n,double amount); // Remove element (return false if not enough resource)
	double get(int n);                // Get amount of resource
	double getdelta(int n);           // Get amount of resource flow in projected 24 hours
	bool enough(int n,double amount); // Is there enough resource?

private:
	double dres_accu[ELEMENTS];	// Delta accumulated during last day
	double dres[ELEMENTS];	    // Real delta during last day
};

#endif